#include <stdio.h>
#include <stdlib.h>
#include <SDL2/SDL.h>
#include <SDL2/SDL_image.h>
#include <SDL2/SDL_mixer.h>
#include "map.h"
#include "personnage.h"
#include "prof.h"
#include "DS.h"
#include "GAME.h"
#include "move.h"
#include "button.h"
#include "timer.h"
#include "window.h"


int main(int argc, char* args[])
{

    /*=======================================*\
    ||                                       ||
    ||             INIT SDL                  ||
    ||                                       ||
    \*=======================================*/

	//Init SDL soft with timer
    if ( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER) != 0)
    {
        printf("Cannot init SDL GAME, Error : %s\n", SDL_GetError());
        return 1;
    }

    // create a new window
    SDL_Window * window = SDL_CreateWindow("INSAGAME",SDL_WINDOWPOS_CENTERED,SDL_WINDOWPOS_CENTERED,WINDOW_WIDTH+2,WINDOW_HEIGHT+2,0);
    if (!window)
    {
        printf("Cannot create new window: %s\n", SDL_GetError());
        SDL_Quit();
        return 1;
    }

    // create a new renderer for graphic sofware
    Uint32 render_flags = SDL_RENDERER_ACCELERATED;
    SDL_Renderer* renderer = SDL_CreateRenderer(window, -1, render_flags);
    if (!renderer)
    {
        printf("Cannot create renderer: %s\n", SDL_GetError());
        SDL_DestroyWindow(window);
        SDL_Quit();
        return 1;
    }

    //Init Music and effects
    Mix_OpenAudio(44100, MIX_DEFAULT_FORMAT, 2, 2048);
    Mix_Music *intro_music = Mix_LoadMUS("../ressource/music.mp3");
    Mix_Music *touch_DS = Mix_LoadMUS("../ressource/touch_DS.wav");
    Mix_Music *lost = Mix_LoadMUS("../ressource/lost.wav");
    Mix_Music *life_lost = Mix_LoadMUS("../ressource/death.wav");
    Mix_Music *victory = Mix_LoadMUS("../ressource/victory.mp3");


    /*=======================================*\
    ||                                       ||
    ||            INIT GAME's FLAGS          ||
    ||                                       ||
    \*=======================================*/

    //FLAG close_request for program
    int close_request = 0;

    //flag to say that the game is beginning => take a random direction for teacher
    int first=1;

    //FLAG FOR DIE, LEVELUP
    int DIE_FLAG=0;
    int LEVELUP_FLAG=0;




    /*=======================================*\
    ||                                       ||
    ||            INIT GAME's ELEMENTS       ||
    ||             AND CHARACTERS            ||
    ||                                       ||
    \*=======================================*/


    //-----------------MAP--------------------//

	//Create the Map and query its texture
	MAP map_game;
    SDL_Texture * texture_map = query_map(&map_game, renderer, window);

    //-----------------HERO--------------------//
	//create a Personnage "hero" and its texture, SDL rect
	Personnage hero;
    SDL_Rect hero_rect;
    SDL_Texture *texture_hero ;

    //-----------------TEACHER--------------------//

	//create a Prof "teacher" and its texture, SDL rect
	Prof teacher;
    SDL_Rect teacher_rect;
    SDL_Texture* texture_teacher;
    //direction random of teacher
    int direction_teacher;


    //-----------------EXAMS--------------------//

	//Create a DS "exam" and query its texture
	DS exam;
    srand(time(0));
    init_DS(&exam,2 + rand() % 29,"../ressource/DS.png");
    SDL_Rect exam_rect;
    SDL_Texture * texture_exam = query_exam(&exam, &exam_rect, renderer, window);

    //-----------------HEARTS--------------------//

    //Create an image of 1,2 and 3 heart for life and query their texture

    SDL_Texture *texture_1heart;
    SDL_Rect heart1_rect;
    texture_1heart = query_heart(&heart1_rect, renderer, window, "../ressource/1heart.png");

    SDL_Texture *texture_2heart;
    SDL_Rect heart2_rect;
    texture_2heart = query_heart(&heart2_rect, renderer, window, "../ressource/2heart.png");

    SDL_Texture *texture_3heart;
    SDL_Rect heart3_rect;
    texture_3heart = query_heart(&heart3_rect, renderer, window, "../ressource/3heart.png");

    //-----------------WINDOW_PLAY--------------------//

    //create new window play containts level, time and id of the current window
	Window window_play;
	init_window(&window_play,-1,-1,INTRO_PAGE);    // init at the window of intro


    //-----------------BUTTONS AND MOUSE--------------------//

    //create and init 3 button of game
	Button button_start_game;
	init_button(&button_start_game,440,556,191,68);

    Button button_replay_win;
    init_button(&button_replay_win,225,580,262,67);

    Button button_replay_lose;
    init_button(&button_replay_lose,225,560,262,67);

    //position of mouse for click
    int mouseX, mouseY;


    //-----------------GAME's-SCORE--------------------//

    //texture for holding the game's score => no init or query now
	SDL_Texture * texture_score;
	SDL_Rect score_rect;
    //game's score variable
    int score=0;

    //-----------------win and lose window image--------------------//

    SDL_Texture *texture_win_page = IMG_LoadTexture(renderer,"../ressource/win.png");

    SDL_Texture *texture_lose_page = IMG_LoadTexture(renderer,"../ressource/lose.png");



    /*=======================================*\
    ||                                       ||
    ||              TIMER                    ||
    ||                                       ||
    \*=======================================*/

    //textur and rect of timer => Init and query in the game !!!
    SDL_Rect timer_rect;
    SDL_Texture * texture_timer;
    //time_remaining of the timer
    int time_remaining=0;



        /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*\
        |%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                                       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|
        |%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%              GAME LOOP                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|
        |%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                                       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|
        \*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/


    while (!close_request)
    {
                                            /*=========================================*\
                                            ||                                         ||
                                            ||              ORIENTATION GRAPH          ||
                                            ||                                         ||
                                            \*=========================================*/


//      #INTRO=======================INTRO_BUTTON_CLICK===============||
//                                                                    ||
//                                                                     V
//      #FAIL_PAGE========FAIL_REPLAY_BUTTON_CLICK=============>     LEVEL     <=======WIN_REPLAY_BUTTON_CLICK=========#WIN_PAGE
//           ^                                                       |   |                                               ^
//           |______________hero's result == LOSE____________________|   |_______________hero's result == WIN____________|


        //Init Background Music
        Mix_Music *background_music = Mix_LoadMUS("../ressource/one_0.mp3");

        switch (window_play.id_window) {    /*Orientation => choose what window to display and to contact with

                                    /*=======================================*\
                                    ||                                       ||
                                    ||              PAGE INTRO               ||
                                    ||                                       ||
                                    \*=======================================*/

            case INTRO_PAGE :
            {
                //load and show the image of intro page
                SDL_Texture *texture_intro_page = IMG_LoadTexture(renderer,"../ressource/gameinstruction.png");
                SDL_RenderCopy(renderer, texture_intro_page, NULL, NULL);
                SDL_RenderPresent(renderer);


                //create process events for start game
                SDL_Event event_button_start_game;

                //Start playing intro music
                Mix_PlayMusic(intro_music,0);

                //-------LOOP INTRO PAGE FOR WAITING BUTTON CLICK--------//

                while(window_play.id_window==INTRO_PAGE){

                    //when dectection of new event
                    while (SDL_PollEvent(&event_button_start_game))
                    {
                        //What type of event ?
                        switch (event_button_start_game.type)
                        {
                            case SDL_QUIT:      //QUIT
                                close_request = 1;            //set the close's flag
                                window_play.id_window=-1;     //for get out of window play 's orientation
                                break;


                            case SDL_MOUSEBUTTONDOWN:     //MOUSE BUTTON CLICK
                                if(event_button_start_game.button.button == SDL_BUTTON_LEFT){

                                    //get the current position of mouse
                                    SDL_GetMouseState(&mouseX,&mouseY);

                                    if(clicked(mouseX,mouseY,&button_start_game)){    //check button clicked

                                        button_start_game.state=0;

                                        //change id window play to "LEVEL", set level = 1, time = 30
                                        init_window(&window_play,1,30,LEVEL);
                                    }
                                }

                        }
                    }
                }

                //Destroy the image, prepare for level 1 => show image of level 1's presentation
                SDL_DestroyTexture(texture_intro_page);
                show_image_in_time(renderer,window,"../ressource/niveau.png",4000);

                //Free Intro Music
                Mix_FreeMusic(intro_music);
                break;
            }



                                    /*=======================================*\
                                    ||                                       ||
                                    ||       PLAYING PAGE WITH LEVELS        ||
                                    ||                                       ||
                                    \*=======================================*/

            case LEVEL:
            {


                init_timer();

                //init hero and query its texture
                init_pers(&hero,3,480,28,23,SPEED,"../ressource/man.png");
                texture_hero = query_hero(&hero,&hero_rect,renderer,window);
                hero.result=PLAYING;

                //init teacher and query its texture
                init_prof(&teacher,350,380,40,25,SPEED,"../ressource/prof.png");
                texture_teacher = query_teacher(&teacher, &teacher_rect, renderer, window);

                //Start playing background music
                Mix_PlayMusic(background_music,0);



                //-------LOOP PLAYING PAGE --------//

                while(hero.result==PLAYING){


                    //-------HERO's PROCESS--------//

                    //create a process events for this playing page
                    SDL_Event event_playing_page;

                    //test key up-down-left-right
                    test_event_hero(&hero,&event_playing_page,&close_request);

                    //move the hero
                    move_hero(&hero,&hero_rect,&teacher,&exam,&map_game,&close_request,&DIE_FLAG,&LEVELUP_FLAG);

                    //-------TIMER PROCESS--------//

                    texture_timer=query_timer(&hero,renderer, &timer_rect, window, window_play.time, &time_remaining);

                    //-------TEACHER PROCESS--------//
					
                    //move the teacher
                    //at the beginning, choose a random direction, else move the teacher
                    if(first==1){
                        srand(time(0));
                        direction_teacher = rand()%4;
                        first=0;
                    }
                    else prof_move(&teacher, &direction_teacher, &teacher_rect, &map_game, &exam);


                    //-------SDL DISPLAY PROCESS--------//

                    // clear the window befor display new things
                    SDL_RenderClear(renderer);

                    // copy the images to the renderer
                    SDL_RenderCopy(renderer, texture_map, NULL, NULL);
                    SDL_RenderCopy(renderer, texture_hero, NULL, &hero_rect);
                    SDL_RenderCopy(renderer, texture_teacher, NULL, &teacher_rect);
                    SDL_RenderCopy(renderer, texture_exam, NULL, &exam_rect);
                    SDL_RenderCopy(renderer, texture_timer, NULL, &timer_rect);


                    //copy heart's image to renderer following hero's life

                    switch (hero.life) {
                        case 1:
                            SDL_RenderCopy(renderer, texture_1heart, NULL, &heart1_rect);
                            break;
                        case 2:
                            SDL_RenderCopy(renderer, texture_2heart, NULL, &heart1_rect);
                            break;
                        case 3:
                            SDL_RenderCopy(renderer, texture_3heart, NULL, &heart2_rect);
                            break;

                    }


                    // display
                    SDL_RenderPresent(renderer);


                    //-------FLAG PROCESS--------//

                    if(DIE_FLAG){        //touch the teacher or timing out

                        //case hero's life = 2 and 3
                        if(hero.life>1){

                            //reset teacher and hero position
                            hero.x_pers = 3;
                            hero.y_pers = 480;
							teacher.x_prof = 350;
                            teacher.y_prof = 380;

                            //decrement hero's life and play the sound effect
                            Mix_PlayMusic(life_lost,0);
                            hero.life--;

                            //show the image of losing life
                            show_image_in_time(renderer,window,"../ressource/lose_heart.png",500);
                        }

                        //case out of life
                        else {
                            hero.result=LOSE;
                        }

                        //reset the die's flag and restart the music
                        DIE_FLAG=0;
                        Mix_PlayMusic(background_music,0);
                    }


                    if(LEVELUP_FLAG)          //when touch the exam
                    {
                        //increment exam took and play the sound effect
                        Mix_PlayMusic(touch_DS,0);
                        SDL_Delay(2000);
                        hero.DS_took++;

                        //random a new position for new exam
                        srand(time(NULL));
                        exam.door = 2 + rand() % 29;
                        exam_number_to_position(&exam);
                        texture_exam = query_exam(&exam, &exam_rect, renderer, window);

                        //what we do next ?
                        switch (window_play.level) {
                            //if level == 1 => set level = 2 => show level 2 presentation
                            case 1:
                            {
                                Mix_PlayMusic(background_music,0);
                                init_window(&window_play,2,20,LEVEL);
                                show_image_in_time(renderer,window,"../ressource/niveau_2.jpeg",4000);
                                break;
                            }
                            //if level == 2 => set level = 3 => show level 3 presentation
                            case 2:
                            {
                                Mix_PlayMusic(background_music,0);
                                init_window(&window_play,3,15,LEVEL);
                                show_image_in_time(renderer,window,"../ressource/niveau_3.jpeg",4000);
                                break;
                            }
                            //if level == 3 => set level = 4 => show level 4 presentation
                            case 3:
                            {
                                Mix_PlayMusic(background_music,0);
                                init_window(&window_play,4,10,LEVEL);
                                show_image_in_time(renderer,window,"../ressource/niveau_4.jpeg",4000);
                                break;
                            }
                            //if level == 4 => set level = 5 => show level 5 presentation
                            case 4:
                            {
                                Mix_PlayMusic(background_music,0);
                                init_window(&window_play,5,5,LEVEL);
                                show_image_in_time(renderer,window,"../ressource/niveau_5.jpeg",4000);
                                break;
                            }
                            //if level == 5 => WIN => set hero's result = WIN
                            case 5:
                                hero.result=WIN;
                                break;

                        }

                        //reset the timer and hero's position
                        init_timer();
                        hero.x_pers = 3;
                        hero.y_pers = 480;

                        //reset LEVELUP_FLAG
                        LEVELUP_FLAG=0;
                    }

                    //-------- Check hero's result--------//

                    if(hero.result==WIN){
                        //set window play id to WIN_PAGE, no time, no level
                        Mix_FreeMusic(background_music);
                        Mix_PlayMusic(victory,0);
                        init_window(&window_play,-1,-1,WIN_PAGE);
                        break;
                    }
                    else if(hero.result==LOSE){
                        //set window play id to FAIL_PAGE, no time, no level
                        Mix_PlayMusic(lost,0);
                        init_window(&window_play,-1,-1,FAIL_PAGE);
                        break;
                    }

                    // wait 17ms before new loop
                    SDL_Delay(17);
                }


                break;
            }



									/*=======================================*\
									||                                       ||
									||               WIN PAGE                ||
									||                                       ||
									\*=======================================*/

            case WIN_PAGE :
            {

                //get and query the score following life, exam and the time remaining
                score = calculer_score(&hero,&time_remaining);
                SDL_Texture *texture_score = query_score(renderer,&score_rect,window, &score);

                //display win page image and score
                SDL_RenderCopy(renderer, texture_win_page, NULL, NULL);
                SDL_RenderCopy(renderer, texture_score, NULL, &score_rect);
                SDL_RenderPresent(renderer);

                //create a process events for this playing page
                SDL_Event event__win_game;

                //-------LOOP INTRO PAGE FOR WAITING BUTTON CLICK--------//

                while(window_play.id_window==WIN_PAGE){

                    //when dectection of new event
                    while (SDL_PollEvent(&event__win_game))
                    {
                        //what type of event ?
                        switch (event__win_game.type)
                        {
                            case SDL_QUIT:              // QUIT
                                close_request = 1;            //set the close flag
                                window_play.id_window=-1;     //for get out of window play 's orientation
                                break;
                            case SDL_MOUSEBUTTONDOWN:     //MOUSE BUTTON CLICK
                                if(event__win_game.button.button == SDL_BUTTON_LEFT){

                                    //get the current position of mouse
                                    SDL_GetMouseState(&mouseX,&mouseY);

                                    if(clicked(mouseX,mouseY,&button_replay_win)){ //check button clicked

                                        button_replay_win.state=0;

                                        //set window play id to "LEVEL", level 1, time = 30
                                        init_window(&window_play,1,30,LEVEL);

                                        //display level 1 presentation
                                        show_image_in_time(renderer,window,"../ressource/niveau.png",4000);
                                    }
                                }

                        }
                    }
                }
                break;
            }


									/*=======================================*\
									||                                       ||
									||               FAIL PAGE                ||
									||                                       ||
									\*=======================================*/

            case FAIL_PAGE :
            {
                //get the score following life, exam and the time remaining
                score = calculer_score(&hero,&time_remaining);
                SDL_Texture *texture_score = query_score(renderer,&score_rect,window, &score);

                //display fail page image and score
                SDL_RenderCopy(renderer, texture_lose_page, NULL, NULL);
                SDL_RenderCopy(renderer, texture_score, NULL, &score_rect);
                SDL_RenderPresent(renderer);


                // process events
                SDL_Event event_button_replay_lose_game;

                //-------LOOP INTRO PAGE FOR WAITING BUTTON CLICK--------//

                while(window_play.id_window==FAIL_PAGE){

                    while (SDL_PollEvent(&event_button_replay_lose_game))
                    {
                        switch (event_button_replay_lose_game.type)
                        {
                            case SDL_QUIT:                  //quit
                                close_request = 1;            //set the close's flag
                                window_play.id_window=-1;     //for get out of window play 's orientation
                                break;
                            case SDL_MOUSEBUTTONDOWN:     //MOUSE BUTTON CLICK
                                if(event_button_replay_lose_game.button.button == SDL_BUTTON_LEFT){

                                    //get the current position of mouse
                                    SDL_GetMouseState(&mouseX,&mouseY);


                                    if(clicked(mouseX,mouseY,&button_replay_lose)){    //check button clicked

                                        button_replay_lose.state=0;

                                        //set window play id to "LEVEL", level 1, time = 30
                                        init_window(&window_play,1,30,LEVEL);

                                        //display level 1 presentation
                                        show_image_in_time(renderer,window,"../ressource/niveau.png",4000);
                                    }
                                }

                        }
                    }
                }
                break;
            }

        }
    }


    /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*\
    |%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                                       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|
    |%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%          AVENGERS: END GAME           %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|
    |%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                                       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|
    \*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/


            //-------------------Destroy Everything-------------//

    SDL_DestroyTexture(texture_map);
    SDL_DestroyTexture(texture_timer);
    SDL_DestroyTexture(texture_1heart);
    SDL_DestroyTexture(texture_2heart);
    SDL_DestroyTexture(texture_3heart);
    SDL_DestroyTexture(texture_hero);
    SDL_DestroyTexture(texture_teacher);
    SDL_DestroyTexture(texture_exam);
    SDL_DestroyTexture(texture_score);
    SDL_DestroyTexture(texture_win_page);
    SDL_DestroyTexture(texture_lose_page);
    SDL_DestroyRenderer(renderer);
    SDL_DestroyWindow(window);
    Mix_FreeMusic(lost);
    Mix_FreeMusic(touch_DS);
    Mix_FreeMusic(life_lost);
    Mix_FreeMusic(victory);
    Mix_CloseAudio();
    SDL_Quit();

    return 0;
}
